//
//  AbstractNetworkOperation.m
//  iSub
//
//  Created by Aaron Brethorst on 5/12/11.
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "AbstractNetworkOperation.h"

#import "UONetworkStatusCoordinator.h"


@interface AbstractNetworkOperation ()
- (void)finish;
@end

@implementation AbstractNetworkOperation
@synthesize url = _url;
@synthesize request = _request;
@synthesize credential = _credential;
@synthesize statusCode = _statusCode;
@synthesize error = _error;
@synthesize isExecuting = _isExecuting;
@synthesize isFinished = _isFinished;

- (id)initWithUrl:(NSURL *)url
{
	return [self initWithRequest:[NSURLRequest requestWithURL:url]];
}

- (id)initWithRequest:(NSURLRequest*)request;
{
	if (!(self = [super init]))
		return nil;
	
    _url = [[request URL] copy];
	_request = [request copy];
	
	return self;
}

- (void)dealloc
{
    [_url release];
	[_request release];
	[_credential release];
    [_connection release];
    [_error release];
    [super dealloc];
}

- (BOOL)isConcurrent
{
    return YES;
}

- (void)start
{
    if (![NSThread isMainThread])
    {
        [self performSelectorOnMainThread:@selector(start) withObject:nil waitUntilDone:NO];
        return;
    }
    
    NSLog(@"operation for <%@> started.", _url);
    
    [self willChangeValueForKey:@"isExecuting"];
    _isExecuting = YES;
    [self didChangeValueForKey:@"isExecuting"];
	
    _connection = [[NSURLConnection alloc] initWithRequest:self.request delegate:self];
	[UONetworkStatusCoordinator addNetworkActivity];
    if (nil == _connection)
	{
        [self finish];
	}
}

- (void)finish
{
    NSLog(@"operation for <%@> finished. "
          @"status code: %d, error: %@",
          _url, _statusCode, _error);
    
	[UONetworkStatusCoordinator removeNetworkActivity];
	
    [_connection release];
    _connection = nil;
    
    [self willChangeValueForKey:@"isExecuting"];
    [self willChangeValueForKey:@"isFinished"];
	
    _isExecuting = NO;
    _isFinished = YES;
	
    [self didChangeValueForKey:@"isExecuting"];
    [self didChangeValueForKey:@"isFinished"];
}

#pragma mark -
#pragma mark NSURLConnection delegate

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{	
    NSHTTPURLResponse * httpResponse = (NSHTTPURLResponse *)response;
    _statusCode = [httpResponse statusCode];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    //no-op, subclasses must implement this.
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    [self finish];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    _error = [error copy];
    [self finish];
}

- (void)connection:(NSURLConnection *)connection didReceiveAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge;
{
	if (self.credential &&
		[challenge previousFailureCount] == 0)
	{
		[[challenge sender] useCredential:self.credential forAuthenticationChallenge:challenge];
	}
	else
	{
		[[challenge sender] continueWithoutCredentialForAuthenticationChallenge:challenge];
	}
}

@end
