//
//  Copyright 2011-2013 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import <AssetsLibrary/ALAssetsLibrary.h>
#import <AssetsLibrary/ALAssetsGroup.h>
#import <AssetsLibrary/ALAsset.h>
#import "U1Logging.h"

#import "FilesAppDelegate.h"

#import "U1Asset.h"
#import "U1AccountManager.h"
#import "U1DataRepository.h"
#import "U1FileNode.h"
#import "U1FileDetailsViewController.h"
#import "U1FilesClient.h"
#import "U1AutoUploadsManager.h"
#import "U1UploadsPoolViewController.h"
#import "U1FolderNode.h"
#import "U1FolderViewController.h"
#import "U1LoginController.h"
#import "U1Node.h"
#import "U1SettingsViewController.h"
#import "U1VolumesViewController.h"
#import "U1Volume.h"
#import "UIAlertView+Blocks.h"
#import "RIButtonItem.h"

@interface FilesAppDelegate () <U1LoginControllerDelegate>
@property (retain) U1FilesClient *filesClient;
@property (retain) U1AutoUploadsManager *autoUploadsManager;
@property (retain) U1LoginController *loginController;
@property (retain) U1DataRepository *dataRepository;
- (void)showVolumes;
- (void)showLoginController;
- (void)attemptVerification;
- (void)prepareTestFlight;
@end


@implementation FilesAppDelegate

@synthesize window, volumesNavController, cameraNavController;
@synthesize filesClient, autoUploadsManager, tabBarController, settingsNavController, loginController, dataRepository;

- (void)dealloc
{
	[window release];
	[volumesNavController release];
	[cameraNavController release];
	[settingsNavController release];
	[filesClient release];
	[autoUploadsManager release];
	[tabBarController release];
	[loginController release];
	[dataRepository release];
    [super dealloc];
}

- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation;
{
	BOOL fromURL = NO;
	
	if ([[url scheme] isEqualToString:@"x-ubuntuone-sso-files"])
	{
		 fromURL = YES;
	}
	return fromURL;
}

- (BOOL)application:(UIApplication *)application handleOpenURL:(NSURL *)url;
{
	return [self application:application openURL:url sourceApplication:nil annotation:nil];
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
	[self prepareTestFlight];
    
    [Crashlytics startWithAPIKey:@"0606692bafe724ed1413548f6211a8557140eade"];
	
	// Migrate any existing old core data repository to the new location.
	NSFileManager *defaultFileManager = [NSFileManager defaultManager];
	if ([defaultFileManager fileExistsAtPath:[[U1DataRepository databaseURLInDirectory:NSCachesDirectory] path]])
	{
		NSLog(@"Need to migrate the old database to the proper location");
		NSURL *oldDatabaseLocation = [U1DataRepository databaseURLInDirectory:NSCachesDirectory];
		NSURL *propertDatabaseLocation = [U1DataRepository databaseURLInDirectory:NSDocumentDirectory];
		NSError *error = nil;
		
		[defaultFileManager moveItemAtURL:oldDatabaseLocation toURL:propertDatabaseLocation error:&error];
	}
	
	[[NSUserDefaults standardUserDefaults] registerDefaults:[NSDictionary dictionaryWithContentsOfFile:
															 [[NSBundle mainBundle] pathForResource:@"default-settings" ofType:@"plist"]]];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didSignOut:) name:U1AccountManagerDidSignOut object:nil];
	
	self.autoUploadsManager = [U1AutoUploadsManager sharedAutoUploadsManager];
	self.filesClient = [U1FilesClient sharedFilesClient];
	self.dataRepository = [U1DataRepository sharedDataRepository];

    [window setFrame:[[UIScreen mainScreen] bounds]];
	[window setRootViewController:tabBarController];
//    [window addSubview:[tabBarController view]];
	[self.window makeKeyAndVisible];

	U1AccountManager *accountManager = [U1AccountManager sharedAccountManager];
	if (![accountManager hasCredentials])
	{
		[self showLoginController];
	}
	else if (![accountManager isVerified])
	{
		[self attemptVerification];
	}
	else
	{
		[self showVolumes];
	}
	
	// XXX: Remove this later, some users still have an app icon badge set
	[[UIApplication sharedApplication] setApplicationIconBadgeNumber:0];

	return YES;
}

- (void)applicationDidBecomeActive:(UIApplication *)application;
{
	U1AccountManager *accountManager = [U1AccountManager sharedAccountManager];
	if (![accountManager hasCredentials])
	{
		[self showLoginController];
	}
	else if (![accountManager isVerified])
	{
		[self attemptVerification];
	}
}

- (void)applicationWillResignActive:(UIApplication *)application;
{
	NSError *err = NULL;
	BOOL success = [self.dataRepository save:&err];
	if(!success){
		CLS_LOG(@"Error saving repository on resigning active: %@", err);
	}
	[[NSUserDefaults standardUserDefaults] synchronize];
	[[UIApplication sharedApplication] setIdleTimerDisabled:NO];
}

- (void)applicationWillTerminate:(UIApplication *)application;
{
	NSError *err = NULL;
	BOOL success = [self.dataRepository save:&err];
	if(!success){
		CLS_LOG(@"Error saving repository on app termination: %@", err);
	}
	[[NSUserDefaults standardUserDefaults] synchronize];
	[[UIApplication sharedApplication] setIdleTimerDisabled:NO];
}

- (void)showVolumes;
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	
	if (![defaults boolForKey:@"asked_auto_upload_on_first_launch"])
	{
		RIButtonItem *cancelButton = [RIButtonItem item];
		cancelButton.label = NSLocalizedString(@"No", @"");
		cancelButton.action = ^
		{
			[defaults setBool:YES forKey:@"asked_auto_upload_on_first_launch"];
			[defaults synchronize];
		};
		
		RIButtonItem *dismissButton = [RIButtonItem item];
		dismissButton.label = NSLocalizedString(@"Yes", @"");
		dismissButton.action = ^
		{
			[defaults setBool:YES forKey:@"auto_upload"];
			[defaults setBool:YES forKey:@"asked_auto_upload_on_first_launch"];
			[defaults synchronize];
		};
		
		UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Auto-Upload", @"")
															message:NSLocalizedString(@"Shall I automatically upload all items in your camera roll?", @"")
												   cancelButtonItem:cancelButton otherButtonItems:dismissButton, nil];
		[alertView show];
		[alertView release];
	}
	
	U1VolumesViewController *volumesViewController = [[[U1VolumesViewController alloc] init] autorelease];
	[self.volumesNavController setViewControllers:[NSArray arrayWithObject:volumesViewController]];
	U1UploadsPoolViewController *assetsViewController = [[[U1UploadsPoolViewController alloc] init] autorelease];
	[self.cameraNavController setViewControllers:[NSArray arrayWithObject:assetsViewController]];
	U1SettingsViewController *settingsViewController = [[[U1SettingsViewController alloc] initWithNibName:@"U1SettingsViewController" bundle:nil] autorelease];
	[self.settingsNavController setViewControllers:[NSArray arrayWithObject:settingsViewController]];
}

- (void)showLoginController;
{
	if (self.loginController == nil)
	{
		self.loginController = [[[U1LoginController alloc] init] autorelease];
		self.loginController.delegate = self;	
		[self.tabBarController presentModalViewController:self.loginController.loginViewController animated:YES];
	}
}

- (void)attemptVerification;
{
	if (self.loginController == nil)
	{
		self.loginController = [[[U1LoginController alloc] init] autorelease];
		self.loginController.delegate = self;
		[self.loginController continueSignUp:self];
	}
}

- (void)loginControllerDidLogin;
{
	[self.loginController setDelegate:nil];
	if ([self.tabBarController modalViewController])
		[self.tabBarController dismissModalViewControllerAnimated:YES];
	self.loginController = nil;
	[self showVolumes];
}

- (void)prepareTestFlight;
{
	NSString *tokenFilePath = [[NSBundle mainBundle] pathForResource:@"U1TestFlightTeamToken" ofType:@"txt"];
	if (!tokenFilePath)
		return;
	
	NSError *error = nil;
	NSString *teamToken = [NSString stringWithContentsOfFile:tokenFilePath usedEncoding:NULL error:&error];
	
	if (teamToken)
	{
		teamToken = [teamToken stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
		[TestFlight takeOff:teamToken];
	}
	else
	{
		NSLog(@"error loading TestFlight team token: %@", error);
	}
}

- (void)didSignOut:(NSNotification*)notification;
{
	UIViewController *blankViewController = [[[UIViewController alloc] init] autorelease];
	[self.volumesNavController setViewControllers:[NSArray arrayWithObject:blankViewController]];
	blankViewController = [[[UIViewController alloc] init] autorelease];
	[self.cameraNavController setViewControllers:[NSArray arrayWithObject:blankViewController]];
	blankViewController = [[[UIViewController alloc] init] autorelease];
	[self.settingsNavController setViewControllers:[NSArray arrayWithObject:blankViewController]];
	
	[self showLoginController];
}

@end
