//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1AccountManager.h"

#import "OAuthConsumer.h"
#import "SFHFKeychainUtils.h"

#define UBUNTU_ONE_SERVICE_NAME @"com.ubuntu.one"
#define UBUNTU_ONE_DUMMY_USER_NAME @"_UBUNTU_ONE_USER_NAME"

#define UBUNTU_ONE_OAUTH_CONSUMER_KEY @"com.ubuntu.one.oauth.consumer"
#define UBUNTU_ONE_OAUTH_TOKEN_KEY @"com.ubuntu.one.oauth.token"

NSString * const U1AccountManagerDidSignOut = @"U1AccountManagerDidSignOut";


@implementation U1AccountManager

+ (U1AccountManager*)sharedAccountManager;
{
	static U1AccountManager *sharedInstance = nil;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		sharedInstance = [[U1AccountManager alloc] init];
	});
	return sharedInstance;
}

- (BOOL)hasCredentials
{
	return !([self oauthConsumer] == nil || [self oauthToken] == nil);
}

- (void)removeCredentials
{
	NSString *consumerKey = [[SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY error:nil] retain];
	if (consumerKey)
	{
		[SFHFKeychainUtils deleteItemForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY error:nil];
		[SFHFKeychainUtils deleteItemForUsername:consumerKey andServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY error:nil];
	}
	
	NSString *token = [[SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY error:nil] retain];
	if (token)
	{
		[SFHFKeychainUtils deleteItemForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY error:nil];
		[SFHFKeychainUtils deleteItemForUsername:token andServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY error:nil];
	}
	
	[consumerKey release], [token release];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:U1AccountManagerDidSignOut object:self];
}

- (void)storeConsumerKey:(NSString*)consumerKey consumerSecret:(NSString*)consumerSecret token:(NSString*)token tokenSecret:(NSString*)tokenSecret;
{
	[SFHFKeychainUtils storeUsername:UBUNTU_ONE_DUMMY_USER_NAME andPassword:consumerKey forServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY updateExisting:YES error:nil];
	[SFHFKeychainUtils storeUsername:consumerKey andPassword:consumerSecret forServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY updateExisting:YES error:nil];
	[SFHFKeychainUtils storeUsername:UBUNTU_ONE_DUMMY_USER_NAME andPassword:token forServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY updateExisting:YES error:nil];
	[SFHFKeychainUtils storeUsername:token andPassword:tokenSecret forServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY updateExisting:YES error:nil];
}

- (BOOL)isVerified;
{
	BOOL verified = NO;
	NSUserDefaults *sud = [NSUserDefaults standardUserDefaults];
	NSString *oauthKey = [[self oauthConsumer] key];
	if (oauthKey)
	{
		NSString *defaultsKey = [@"U1SSOVerified:" stringByAppendingString:oauthKey];
		verified = [sud boolForKey:defaultsKey];
	}
	return verified; // [[NSUserDefaults standardUserDefaults] boolForKey:[@"U1SSOVerified:" stringByAppendingString:[[self oauthConsumer] key]]];
}

- (void)setVerified:(BOOL)verified;
{
	NSUserDefaults *sud = [NSUserDefaults standardUserDefaults];
	NSString *oauthKey = [[self oauthConsumer] key];
	if (oauthKey)
	{
		NSString *defaultsKey = [@"U1SSOVerified:" stringByAppendingString:oauthKey];
		[sud setBool:verified forKey:defaultsKey];
	}
}

- (OAConsumer*)oauthConsumer;
{
	NSString *consumerKey = [SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY error:nil];
	if (!consumerKey)
		return nil;
	
	NSString *consumerSecret = [SFHFKeychainUtils getPasswordForUsername:consumerKey andServiceName:UBUNTU_ONE_OAUTH_CONSUMER_KEY error:nil];
	if (!consumerSecret)
		return nil;
	
	return [[[OAConsumer alloc] initWithKey:consumerKey secret:consumerSecret] autorelease];
}

- (OAToken*)oauthToken;
{
	NSString *tokenKey = [SFHFKeychainUtils getPasswordForUsername:UBUNTU_ONE_DUMMY_USER_NAME andServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY error:nil];
	if (!tokenKey)
		return nil;
	
	NSString *tokenSecret = [SFHFKeychainUtils getPasswordForUsername:tokenKey andServiceName:UBUNTU_ONE_OAUTH_TOKEN_KEY error:nil];
	if (!tokenSecret)
		return nil;
	
	return [[[OAToken alloc] initWithKey:tokenKey secret:tokenSecret] autorelease];
}


@end
