//
//  Copyright 2011-2013 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1AssetRepresenationDataProvider.h"

#import <AssetsLibrary/AssetsLibrary.h>
#import <MobileCoreServices/MobileCoreServices.h>

#import "U1Asset.h"
#import "U1DataRepository.h"

@implementation U1AssetRepresenationDataProvider
{
	ALAssetsLibrary *library;
}

@synthesize assetID;

- (id)init;
{
	self = [super init];
	if (self == nil)
		return nil;
	library = [[ALAssetsLibrary alloc] init];
	return self;
}

- (void)dealloc;
{
	[library release];
	[super dealloc];
}

- (BOOL)isMagic;
{
	return NO;
}

- (void)prepareDataStreamWithBlock:(void(^)(NSInputStream *dataStream, NSString *mimeType, NSUInteger length, NSError *error))block;
{
	__block NSURL *url;
	[[U1DataRepository sharedDataRepository] dispatchBlockWithManagedObjectContext:^(NSManagedObjectContext *context) {
		U1Asset *asset = (U1Asset *)[context objectWithID:self.assetID];
		url = [NSURL URLWithString:asset.url];
	}];

	[library assetForURL:url resultBlock:^(ALAsset *asset) {
		
		if (asset == nil)
		{
			if (block)
			{
				NSError *error = [NSError errorWithDomain:@"nonexistentAsset" code:1 userInfo:nil];
				block(nil, nil, 0, error);
			}
		}
		else
		{
			ALAssetRepresentation *representation = [asset defaultRepresentation];
			uint8_t *buffer = (uint8_t*)malloc(representation.size);
			NSUInteger length = [representation getBytes:buffer fromOffset:0 length:representation.size error:nil];
			NSData *imageData = [NSData dataWithBytesNoCopy:buffer length:length freeWhenDone:YES];
			NSString *mimeType = [(id)UTTypeCopyPreferredTagWithClass((CFStringRef)[representation UTI], kUTTagClassMIMEType) autorelease];
			NSInputStream *dataStream = [NSInputStream inputStreamWithData:imageData];
			if (block)
				block(dataStream, mimeType, length, nil);
		}
		
	} failureBlock:^(NSError *error) {
		if (block)
			block(nil, nil, 0, error);
	}];
}

@end
