//
//  Copyright 2011-2013 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import <Foundation/Foundation.h>

@class NSFetchedResultsController;
@class U1Node, U1FileNode, U1FolderNode, U1NodeChildrenResultSet, U1Volume, U1LocalFileInfo;
@class U1DataRepository, U1FilesService;

extern NSString * const U1FilesClientNodeStatusChangedNotification;

typedef enum U1DataSourceType {
	U1DataSourceLocal,
	U1DataSourceRemote,
} U1DataSourceType;


@interface U1FilesClient : NSObject
@property (retain) U1DataRepository *dataRepository;
@property (retain) U1FilesService *filesService;

+ (U1FilesClient *)sharedFilesClient;
- (id)accountInfoWithCompletionBlock:(void(^)(NSDictionary *accountInfo, NSError *error))completionBlock;
- (id)volumesWithResultsBlock:(void(^)(U1Volume *rootVolume, NSFetchedResultsController *cloudFoldersResultsController, U1DataSourceType resultsDataSourceType))resultsBlock errorBlock:(void(^)(NSError *error))errorBlock;
- (id)fetchFolderWithResourcePath:(NSString*)nodePath resultsBlock:(void(^)(U1Node *node, NSFetchedResultsController *childrenResultsController, U1DataSourceType resultsDataSourceType))resultsBlock errorBlock:(void(^)(NSError *error))errorBlock;
- (id)fetchFileWithResourcePath:(NSString*)nodePath completionBlock:(void(^)(U1Node *node, NSError *error))completionBlock;
- (id)renameNode:(U1Node*)node toName:(NSString*)newName completionBlockOnMainQueue:(void(^)(U1Node *node, NSError *error))completionBlock;
- (id)deleteNode:(U1Node*)node completionBlock:(void(^)(NSError *error))completionBlock;
- (id)uploadAssetWithID:(NSManagedObjectID*)assetID withPriority:(NSOperationQueuePriority)priority progressBlock:(void(^)(long long bytesUploaded, long long totalBytes))progressBlock completionBlockOnMainQueue:(void(^)(U1FileNode *updatedNode, NSError *error))completionBlock;
- (id)publishNode:(U1FileNode*)node completionBlockOnMainQueue:(void(^)(U1FileNode *node, NSError *error))completionBlock;
- (id)unpublishNode:(U1FileNode*)node completionBlockOnMainQueue:(void(^)(U1FileNode *node, NSError *error))completionBlock;
- (id)createFolderNamed:(NSString*)name inFolder:(U1FolderNode*)folderNode completionBlockOnMainQueue:(void(^)(U1FolderNode *newFolderNode, NSError *error))completionBlock;
- (id)createVolumeAtPath:(NSString*)folderPath completionBlockOnMainQueue:(void(^)(U1Volume *volume, NSError *error))completionBlock;
- (BOOL)isNodeSavedLocally:(U1FileNode*)node;
- (U1LocalFileInfo*)localInfoForNode:(U1FileNode*)node;
- (U1LocalFileInfo*)downloadFileForNode:(U1FileNode*)node completionBlock:(void(^)(NSError *error))completionBlock;
- (BOOL)isOperating;
- (BOOL)isPending:(U1FileNode *)fileNode;
- (BOOL)isUploading:(U1FileNode *)fileNode;
- (int)queueCount;
- (U1LocalFileInfo*)fileInfoForNode:(U1FileNode*)fileNode;
@end
