//
//  Copyright 2011-2013 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import <Foundation/Foundation.h>

@class U1Node, U1FileNode, U1FolderNode;


@protocol U1UploadDataProvider <NSObject>
@property (retain) NSManagedObjectID *assetID;
- (void)prepareDataStreamWithBlock:(void(^)(NSInputStream *dataStream, NSString *mimeType, NSUInteger length, NSError *error))block;
- (BOOL)isMagic;
@end


@interface U1FilesService : NSObject

+ (U1FilesService *)sharedFilesService;

- (id)accountInfoWithCompletionBlockOnMainQueue:(void(^)(NSDictionary *accountInfo, NSError *error))completionBlock;
- (id)volumeInfoWithCompletionBlockOnMainQueue:(void(^)(NSArray *volumeInfos, NSError *error))completionBlock;
- (id)infoForNodeAtResourcePath:(NSString*)resourcePath includeChildren:(BOOL)includeChildren completionBlockOnMainQueue:(void(^)(NSDictionary *node, NSError *error))completionBlock;
- (id)infoForNode:(U1Node*)node includeChildren:(BOOL)includeChildren completionBlockOnMainQueue:(void(^)(NSDictionary *node, NSError *error))completionBlock;
- (id)contentForNode:(U1FileNode*)node progressBlock:(void(^)(long long bytesReceived, long long bytesExpected))progressBlock completionBlock:(void(^)(NSURL *contentURL))completionBlock;
- (id)uploadContentData:(id<U1UploadDataProvider>)contentData progressBlock:(void(^)(long long bytesUploaded, long long totalBytes))progressBlock completionBlockOnMainQueue:(void(^)(NSDictionary *updatedNodeInfo, NSError *error))completionBlock;
- (id)moveNodeAtResourcePath:(NSString*)resourcePath toPath:(NSString*)newPath withCompletionBlockOnMainQueue:(void(^)(NSDictionary *nodeInfo, NSError *error))completionBlock;
- (id)deleteNodeAtResourcePath:(NSString*)resourcePath completionBlockOnMainQueue:(void(^)(NSError *error))completionBlock;
- (id)publishNode:(U1FileNode*)node completionBlockOnMainQueue:(void(^)(U1FileNode *node, NSError *error))completionBlock;
- (id)unpublishNode:(U1FileNode*)node completionBlockOnMainQueue:(void(^)(U1FileNode *node, NSError *error))completionBlock;
- (id)createFolderAtPath:(NSString*)folderPath completionBlockOnMainQueue:(void(^)(NSDictionary *nodeInfo, NSError *error))completionBlock;
- (id)createVolumeAtPath:(NSString*)folderPath completionBlockOnMainQueue:(void(^)(NSDictionary *volumeInfo, NSError *error))completionBlock;

@end
