//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import <Foundation/Foundation.h>

@protocol U1HTTPRequestOperationDataCollector;
@protocol U1HTTPRequestOperationDelegate;


/*!
 A base class for implementing HTTP request operations.
 
 This class schedules the connection on the runloop for
 the thread which creates the operation.
 */
@interface U1HTTPRequestOperation : NSOperation

@property (retain) NSURLCredential *credential;
@property (readonly, copy) NSHTTPURLResponse *response;
@property (readonly, retain) NSError *error;

- (id)initWithRequest:(NSURLRequest*)request dataCollector:(id<U1HTTPRequestOperationDataCollector>)dataCollector;

@end


/*!
 The methods in the UOHTTPRequestOperationDataCollector protocol allow an
 adopting class to handle data received by a UOHTTPRequestOperation.
 */
@protocol U1HTTPRequestOperationDataCollector <NSObject>
- (void)httpRequestOperation:(U1HTTPRequestOperation*)httpOperation willExpectDataOfLength:(long long)length;
- (void)httpRequestOperation:(U1HTTPRequestOperation *)httpOperation didReceiveData:(NSData*)data;
- (void)httpRequestOperationDidFinishLoadingData:(U1HTTPRequestOperation*)httpOperation;
- (void)httpRequestOperation:(U1HTTPRequestOperation*)httpOperation didFailWithError:(NSError*)error;
@end


@interface U1HTTPRequestOperationInMemoryDataCollector : NSObject <U1HTTPRequestOperationDataCollector>
@property (readonly, retain) NSData *data;
@end


@interface U1HTTPRequestOperationFileDataCollector : NSObject <U1HTTPRequestOperationDataCollector>
@property (readonly, copy) NSURL *fileURL;
- (id)initWithURL:(NSURL*)url;
@end


@interface U1HTTPRequestOperationReportingDataCollector : NSObject <U1HTTPRequestOperationDataCollector>
@property (copy) void (^progressBlock)(long long receivedBytes, long long expectedBytes);
- (id)initWithDataCollector:(id<U1HTTPRequestOperationDataCollector>)nextCollector;
@end


@protocol U1HTTPRequestOperationDelegate <NSObject>
- (NSURLCredential*)credentialForRequest;
@end
