//
//  Copyright 2012-2013 Canonical Ltd.
//
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1MagicUploadDataProvider.h"

#import <CommonCrypto/CommonDigest.h>
#import <AssetsLibrary/AssetsLibrary.h>

#import "U1Asset.h"
#import "U1DataRepository.h"

@implementation U1MagicUploadDataProvider
{
	ALAssetsLibrary *library;
}

@synthesize assetID;

+ (NSString *)hexStringFromData:(NSData *)data{
	NSMutableString *hexString = [NSMutableString stringWithCapacity:2 * [data length]];
	const unsigned char *dataBytes = [data bytes];
	for (NSInteger idx = 0; idx < [data length]; ++idx) {
		[hexString appendFormat:@"%02x", dataBytes[idx]];
	}
	return hexString;
}

- (id)init;
{
	self = [super init];
	if (self == nil)
		return nil;
	library = [[ALAssetsLibrary alloc] init];
	self.hasher = [[[U1Hasher alloc] init] autorelease];
	return self;
}

- (void)dealloc;
{
	[library release];
	[super dealloc];
}

- (BOOL)isMagic;
{
	return YES;
}

- (void)prepareDataStreamWithBlock:(void(^)(NSInputStream *dataStream, NSString *mimeType, NSUInteger length, NSError *error))block;
{
	__block NSDictionary *hashDict = nil;
	__block U1Asset *asset;
	[[U1DataRepository sharedDataRepository] dispatchBlockWithManagedObjectContext:^(NSManagedObjectContext *context) {
		asset = (U1Asset *)[context objectWithID:self.assetID];
		
		if (asset == nil){
			if (block){
				NSError *error = [NSError errorWithDomain:@"nonexistentAsset" code:1 userInfo:nil];
				block(nil, nil, 0, error);
			}
		}
		
		if (asset.contentHash == nil) {
			
			NSURL *asset_url = [NSURL URLWithString:asset.url];
			hashDict = [self.hasher synchronouslyGetHashesForAssetURL:asset_url];
			
			if (asset == nil){
				if (block){
					NSError *error = [NSError errorWithDomain:@"nonexistentAsset" code:1 userInfo:nil];
					block(nil, nil, 0, error);
				}
			}
		}
		
		asset.contentHash = [hashDict objectForKey:@"contentHash"];
		asset.magicHash = [hashDict objectForKey:@"magicHash"];
		
		NSError *err;
		if(![context save:&err]){
			NSLog(@"error saving hashes to asset %@\n error is %@", asset, err);
		}
	}];
	
	NSString *contentHashString = [U1MagicUploadDataProvider hexStringFromData:asset.contentHash];
	NSString *magicHashString = [U1MagicUploadDataProvider hexStringFromData:asset.magicHash];
	magicHashString = [NSString stringWithFormat:@"sha1:%@", magicHashString];
	contentHashString = [NSString stringWithFormat:@"sha1:%@", contentHashString];
	
	NSDictionary *magicDict = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:@"file", contentHashString, magicHashString, nil]
														  forKeys:[NSArray arrayWithObjects:@"kind", @"hash", @"magic_hash", nil]];
	NSData *magicDictData = [NSJSONSerialization dataWithJSONObject:magicDict options:0 error:nil];
	NSString *magicDictAsJSON = [[[NSString alloc] initWithData:magicDictData encoding:NSUTF8StringEncoding] autorelease];
	NSInputStream *dataStream = [NSInputStream inputStreamWithData:[magicDictAsJSON dataUsingEncoding:NSUTF8StringEncoding]];
	
	if (block)
		block(dataStream, @"application/json", [[magicDictAsJSON dataUsingEncoding:NSUTF8StringEncoding] length], nil);
}

@end
