//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1Node.h"

#import "U1FileNode.h"
#import "U1FolderNode.h"


@implementation U1Node

@dynamic isFile, isFolder;

+ (id)nodeWithJSONProperties:(NSDictionary*)jsonProperties;
{
	Class nodeClass = Nil;
	NSString *kind = [jsonProperties objectForKey:@"kind"];
	if ([@"directory" isEqualToString:kind])
		nodeClass = [U1FolderNode class];
	else if ([@"file" isEqualToString:kind])
		nodeClass = [U1FileNode class];
	
	return [[[nodeClass alloc] initWithJSONProperties:jsonProperties] autorelease];
}

+ (NSDateFormatter*)jsonDateFormatter;
{
	static NSDateFormatter *jsonDateFormatter = nil;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		jsonDateFormatter = [[NSDateFormatter alloc] init];
		[jsonDateFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];
		[jsonDateFormatter setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss'Z'"];
	});
	return jsonDateFormatter;
}

- (id)initWithJSONProperties:(NSDictionary*)jsonProperties;
{
	if (!(self = [super init]))
		return nil;
	
	[self updatePropertiesFromJSONDictionary:jsonProperties];

	return self;
}

- (void)updatePropertiesFromJSONDictionary:(NSDictionary*)jsonDictionary;
{
	self.kind = [jsonDictionary nonNullValueForKey:@"kind"];
	self.contentPath = [jsonDictionary nonNullValueForKey:@"content_path"];
	self.resourcePath = [jsonDictionary nonNullValueForKey:@"resource_path"];
	self.volumePath = [jsonDictionary nonNullValueForKey:@"volume_path"];
	self.path = [jsonDictionary nonNullValueForKey:@"path"];
	self.key = [jsonDictionary nonNullValueForKey:@"key"];
	self.generation = [jsonDictionary nonNullValueForKey:@"generation"];
	self.generationCreated = [jsonDictionary nonNullValueForKey:@"generation_created"];
	
//	NSDate *date = [[U1Node jsonDateFormatter] dateFromString:[jsonDictionary nonNullValueForKey:@"when_created"]];
//	self.creationDate = date;
//	date = [[U1Node jsonDateFormatter] dateFromString:[jsonDictionary nonNullValueForKey:@"when_changed"]];
//	self.modificationDate = date;
	NSDate *creationDate = [[U1Node jsonDateFormatter] dateFromString:[jsonDictionary nonNullValueForKey:@"when_created"]];
	self.creationDate = creationDate;
	NSDate *modificationDate = [[U1Node jsonDateFormatter] dateFromString:[jsonDictionary nonNullValueForKey:@"when_changed"]];
	self.modificationDate = modificationDate;
}

- (NSString *)name;
{
	return [self.contentPath lastPathComponent];
}

- (BOOL)isFile;
{
	return NO;
}

- (BOOL)isFolder;
{
	return NO;
}

@end
