//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1SSOResponse.h"


NSString * const UOSingleSignOnErrorDomain = @"UOSingleSignOnErrorDomain";
NSString * const UOSingleSignOnErrorKey = @"UOSingleSignOnErrorKey";


@implementation U1SSOResponse

@synthesize status, message, errors;

- (void)dealloc;
{
	self.status = nil;
	self.message = nil;
	self.errors = nil;
	[super dealloc];
}

- (NSString *)description;
{
	return [NSString stringWithFormat:@"<%@: %p; %@>", [self class], self, [self dictionaryWithValuesForKeys:[NSArray arrayWithObjects:@"status", @"message", @"errors", nil]]];
}

- (id)parseFromJSONDictionary:(NSDictionary *)dictionary;
{
	id errorsObject = [dictionary objectForKey:@"errors"];
	NSMutableArray *parsedErrors = [NSMutableArray array];
	if ([errorsObject isKindOfClass:[NSDictionary class]])
	{
		NSDictionary *errorsDict = (id)errorsObject;
		[errorsDict enumerateKeysAndObjectsUsingBlock:^(NSString *errorKey, NSArray *errorsForKey, BOOL *stop) {
			[errorsForKey enumerateObjectsUsingBlock:^(NSString *errorString, NSUInteger idx, BOOL *stop) {
				NSDictionary *userInfo =
				[NSDictionary dictionaryWithObjectsAndKeys:
				 errorString, NSLocalizedDescriptionKey,
				 errorKey, UOSingleSignOnErrorKey,
				 nil];
				NSError *error = [NSError errorWithDomain:UOSingleSignOnErrorDomain code:0 userInfo:userInfo];
				[parsedErrors addObject:error];
			}];
		}];
	}
	else if ([errorsObject isKindOfClass:[NSArray class]])
	{
		NSArray *errorsArray = (id)errorsObject;
		[errorsArray enumerateObjectsUsingBlock:^(NSString *errorString, NSUInteger idx, BOOL *stop) {
			NSDictionary *userInfo =
			[NSDictionary dictionaryWithObjectsAndKeys:
			 errorString, NSLocalizedDescriptionKey,
			 nil];
			NSError *error = [NSError errorWithDomain:UOSingleSignOnErrorDomain code:0 userInfo:userInfo];
			[parsedErrors addObject:error];
		}];
	}
	self.errors = parsedErrors;
	NSMutableDictionary *otherProperties = [[dictionary mutableCopy] autorelease];
	[otherProperties removeObjectForKey:@"errors"];
	return [super parseFromJSONDictionary:otherProperties];
}

@end
