//
//  Copyright 2012 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1SSOSignUpViewController.h"

#import "U1SingleSignOnProvider.h"
#import "U1SSOCaptchaResponse.h"


enum {
	U1SSOSignUpFieldFullName = 1,
	U1SSOSignUpFieldEmail,
	U1SSOSignUpFieldPassword,
	U1SSOSignUpFieldCaptcha,
};
typedef NSUInteger U1SSOSignUpField;


@interface U1SSOSignUpViewController ()
@property (nonatomic, retain, readonly) IBOutlet UIScrollView *scrollView;

@property (nonatomic, retain, readonly) IBOutlet UITextField *fullNameTextField;
@property (nonatomic, retain, readonly) IBOutlet UILabel *fullNameLabel;
@property (nonatomic, retain, readonly) IBOutlet UIImageView *fullNameErrorIndicator;
@property (nonatomic, retain, readonly) IBOutlet UILabel *fullNameErrorLabel;

@property (nonatomic, retain, readonly) IBOutlet UITextField *emailTextField;
@property (nonatomic, retain, readonly) IBOutlet UILabel *emailLabel;
@property (nonatomic, retain, readonly) IBOutlet UIImageView *emailErrorIndicator;
@property (nonatomic, retain, readonly) IBOutlet UILabel *emailErrorLabel;

@property (nonatomic, retain, readonly) IBOutlet UITextField *passwordTextField;
@property (nonatomic, retain, readonly) IBOutlet UILabel *passwordLabel;
@property (nonatomic, retain, readonly) IBOutlet UIImageView *passwordErrorIndicator;
@property (nonatomic, retain, readonly) IBOutlet UILabel *passwordErrorLabel;

@property (nonatomic, retain, readonly) IBOutlet UIImageView *captchaImageView;
@property (nonatomic, retain, readonly) IBOutlet UITextField *captchaTextField;
@property (nonatomic, retain, readonly) IBOutlet UILabel *captchaLabel;
@property (nonatomic, retain, readonly) IBOutlet UIImageView *captchaErrorIndicator;
@property (nonatomic, retain, readonly) IBOutlet UILabel *captchaErrorLabel;

@property (nonatomic, retain, readonly) IBOutlet UIActivityIndicatorView *captchaLoadingIndicator;
@property (copy) NSString *captchaId;

@property (nonatomic, assign) UITextField *activeTextField;
- (BOOL)validateFields;
- (void)keyboardWasShown:(NSNotification *)notification;
- (void) keyboardWillHide:(NSNotification *)notification;
@end


@implementation U1SSOSignUpViewController

@synthesize ssoProvider, signUpAction;
@synthesize scrollView;
@synthesize fullNameTextField, fullNameLabel, fullNameErrorIndicator, fullNameErrorLabel;
@synthesize emailTextField, emailLabel, emailErrorIndicator, emailErrorLabel;
@synthesize passwordTextField, passwordLabel, passwordErrorIndicator, passwordErrorLabel;
@synthesize captchaImageView, captchaTextField, captchaLabel, captchaErrorIndicator, captchaLoadingIndicator, captchaErrorLabel;
@synthesize captchaId;
@synthesize activeTextField;

- (void)dealloc;
{
	[ssoProvider release];
	[signUpAction release];
	[scrollView release];
	[fullNameTextField release];
	[fullNameLabel release];
	[fullNameErrorIndicator release];
	[fullNameErrorLabel release];
	[emailTextField release];
	[emailLabel release];
	[emailErrorIndicator release];
	[emailErrorLabel release];
	[passwordTextField release];
	[passwordLabel release];
	[passwordErrorIndicator release];
	[passwordErrorLabel release];
	[captchaImageView release];
	[captchaTextField release];
	[captchaLabel release];
	[captchaErrorIndicator release];
	[captchaLoadingIndicator release];
	[captchaErrorLabel release];
	[captchaId release];
	[super dealloc];
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidUnload;
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad;
{
	UITapGestureRecognizer *captchaTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(reloadCaptcha:)];
	[self.captchaImageView addGestureRecognizer:captchaTap];
	[captchaTap release];
	
	[self reloadCaptcha:nil];

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(keyboardWasShown:)
												 name:UIKeyboardDidShowNotification
											   object:nil];

	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(keyboardWillHide:)
												 name:UIKeyboardWillHideNotification
											   object:nil];
}

- (void)keyboardWasShown:(NSNotification *)notification;
{
	// Step 1: Get the size of the keyboard.
	CGSize keyboardSize = [[[notification userInfo] objectForKey:UIKeyboardFrameBeginUserInfoKey] CGRectValue].size;

	// Step 2: Adjust the bottom content inset of your scroll view by the keyboard height.
	UIEdgeInsets contentInsets = UIEdgeInsetsMake(0.0, 0.0, keyboardSize.height, 0.0);
	self.scrollView.contentInset = contentInsets;
	self.scrollView.scrollIndicatorInsets = contentInsets;

	// Step 3: Scroll the target text field into view.
	CGRect aRect = self.view.frame;
	aRect.size.height -= keyboardSize.height;
	if (!CGRectContainsPoint(aRect, activeTextField.frame.origin) ) {
		CGPoint scrollPoint = CGPointMake(0.0, activeTextField.frame.origin.y - (keyboardSize.height-15));
		[self.scrollView setContentOffset:scrollPoint animated:YES];
	}
}

- (void) keyboardWillHide:(NSNotification *)notification;
{
	UIEdgeInsets contentInsets = UIEdgeInsetsZero;
	self.scrollView.contentInset = contentInsets;
	self.scrollView.scrollIndicatorInsets = contentInsets;
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
	self.activeTextField = textField;
}

- (void)textFieldDidEndEditing:(UITextField *)textField
{
	self.activeTextField = nil;
}

- (IBAction)reloadCaptcha:(id)sender;
{
	[self.captchaImageView setImage:nil];
	[self.captchaLoadingIndicator setHidden:NO];
	[self.ssoProvider requestNewCaptchaWithCompletionHandlerOnMainQueue:^(U1SSOCaptchaResponse *response, NSError *error) {
		if(response == nil){
			[self.captchaImageView setImage:[UIImage imageNamed:@"indicator_input_error"]];
		}else{
			self.captchaId = response.captchaId;
			NSData *imageData = [NSData dataWithContentsOfURL:[NSURL URLWithString:response.imageURI]];
			UIImage *captchaImage = [UIImage imageWithData:imageData];
			[self.captchaImageView setImage:captchaImage];
		}
		[self.captchaLoadingIndicator setHidden:YES];
	}];
}

- (IBAction)signUp:(id)sender;
{
	if (![self validateFields])
		return;
	
	if (signUpAction)
	{
		NSString *fullName = [self.fullNameTextField text] ?: @"";
		NSString *email = [self.emailTextField text];
		NSString *password = [self.passwordTextField text];
		NSString *captchaResponse = [self.captchaTextField text];
		signUpAction(fullName, email, password, self.captchaId, captchaResponse);
	}
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField;
{
	NSInteger tag = [textField tag];
	++tag;
	
	if (tag > U1SSOSignUpFieldCaptcha)
	{
		[textField resignFirstResponder];
		return YES;
	}
	
	UIView *nextView = [self.view viewWithTag:tag];
	[nextView becomeFirstResponder];

	// this nastiness is so that the captcha image is still visible
	// if we let the scrollview do its magic, the text field ends up centered
	// and the user can't read the captcha image
	if (tag == U1SSOSignUpFieldCaptcha)
	{
		CGRect rect = nextView.frame;
		rect = [scrollView convertRect:rect fromView:nextView.superview];
		rect.origin.y -= 100;
		rect.size.height += 120;
		[scrollView scrollRectToVisible:rect animated:YES];
	}
		
	return YES;
}

- (BOOL)validateFields;
{
	BOOL valid = YES;
	BOOL didScroll = NO;
	[self.emailErrorLabel setText:@""];
	[self.passwordErrorLabel setText:@""];
	[self.captchaErrorLabel setText:@""];
	if ([[self.emailTextField text] length] < 1)
	{
		valid = NO;
		[self.emailErrorIndicator setHidden:NO];
		[self.emailLabel setTextColor:[UIColor colorWithRed:221.f / 255.f green:72.f / 255.f blue:20.f / 255.f alpha:1.f]];
		if (!didScroll)
		{
			[self.scrollView scrollRectToVisible:[self.emailTextField bounds] animated:YES];
			didScroll = YES;
		}
	}
	else
	{
		[self.emailErrorIndicator setHidden:YES];
		[self.emailLabel setTextColor:[UIColor blackColor]];
	}
	if ([[self.passwordTextField text] length] < 1)
	{
		valid = NO;
		[self.passwordErrorIndicator setHidden:NO];
		[self.passwordLabel setTextColor:[UIColor colorWithRed:221.f / 255.f green:72.f / 255.f blue:20.f / 255.f alpha:1.f]];
		if (!didScroll)
		{
			[self.scrollView scrollRectToVisible:[self.passwordTextField bounds] animated:YES];
			didScroll = YES;
		}
	}
	else
	{
		[self.passwordErrorIndicator setHidden:YES];
		[self.passwordLabel setTextColor:[UIColor blackColor]];
	}
	if ([[self.captchaTextField text] length] < 1)
	{
		valid = NO;
		[self.captchaErrorIndicator setHidden:NO];
		[self.captchaLabel setTextColor:[UIColor colorWithRed:221.f / 255.f green:72.f / 255.f blue:20.f / 255.f alpha:1.f]];
		if (!didScroll)
		{
			[self.scrollView scrollRectToVisible:[self.captchaTextField bounds] animated:YES];
			didScroll = YES;
		}
	}
	else
	{
		[self.captchaErrorIndicator setHidden:YES];
		[self.captchaLabel setTextColor:[UIColor blackColor]];
	}
	return valid;
}

- (void)presentErrors:(NSArray *)errors;
{
	BOOL didScroll = NO;
	for (NSError *error in errors)
	{
		NSString *key = [[error userInfo] objectForKey:@"UOSingleSignOnErrorKey"];
		if ([@"email" isEqualToString:key])
		{
			[self.emailErrorIndicator setHidden:NO];
			[self.emailLabel setTextColor:[UIColor colorWithRed:221.f / 255.f green:72.f / 255.f blue:20.f / 255.f alpha:1.f]];
			[self.emailErrorLabel setText:[error localizedDescription]];
			if (!didScroll)
			{
				[self.scrollView scrollRectToVisible:[self.emailTextField bounds] animated:YES];
				didScroll = YES;
			}
		}
		else if ([@"__all__" isEqualToString:key])
		{
			[self.captchaErrorIndicator setHidden:NO];
			[self.captchaLabel setTextColor:[UIColor colorWithRed:221.f / 255.f green:72.f / 255.f blue:20.f / 255.f alpha:1.f]];
			if (!didScroll)
			{
				[self.scrollView scrollRectToVisible:[self.captchaTextField bounds] animated:YES];
				didScroll = YES;
			}
		}
		else if ([@"password" isEqualToString:key])
		{
			[self.passwordErrorIndicator setHidden:NO];
			[self.passwordLabel setTextColor:[UIColor colorWithRed:221.f / 255.f green:72.f / 255.f blue:20.f / 255.f alpha:1.f]];
			if (!didScroll)
			{
				[self.scrollView scrollRectToVisible:[self.passwordTextField bounds] animated:YES];
				didScroll = YES;
			}
		}
	}
}

@end
