//
//  Copyright 2012 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1ScrollView.h"


@implementation U1ScrollView

@dynamic contentView;

- (void)u1_init;
{
	NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
	[notificationCenter addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
	[notificationCenter addObserver:self selector:@selector(keyboardDidHide:) name:UIKeyboardDidHideNotification object:nil];
}

- (id)initWithCoder:(NSCoder *)aDecoder;
{
	self = [super initWithCoder:aDecoder];
	if (self == nil)
		return nil;
	[self u1_init];
	return self;
}

- (id)initWithFrame:(CGRect)frame;
{
	self = [super initWithFrame:frame];
	if (self == nil)
		return nil;
	[self u1_init];
	return self;
}

- (void)dealloc;
{
	NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
	[notificationCenter removeObserver:self name:UIKeyboardWillShowNotification object:nil];
	[notificationCenter removeObserver:self name:UIKeyboardDidHideNotification object:nil];
	[super dealloc];
}

- (UIView*)contentView;
{
	return [self.subviews lastObject];
}

- (void)setContentView:(UIView*)contentView;
{
	[self addSubview:contentView];
	[self setContentSize:contentView.bounds.size];
}

- (void)keyboardWillShow:(NSNotification*)notification;
{
	CGRect keyboardFrame = [[[notification userInfo] objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
	self.contentInset = UIEdgeInsetsMake(0, 0, keyboardFrame.size.height, 0);
}

- (void)keyboardDidHide:(NSNotification*)notification;
{
	self.contentInset = UIEdgeInsetsZero;
}

@end
