//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1Volume.h"
#import "U1FolderNode.h"
#import "U1DataRepository.h"


@implementation U1Volume

- (void)updatePropertiesFromJSONDictionary:(NSDictionary *)jsonDictionary;
{
	[super updatePropertiesFromJSONDictionary:jsonDictionary];
	self.nodePath = [jsonDictionary objectForKey:@"node_path"];
	// override node kind, either root or udf
	self.kind = [jsonDictionary nonNullValueForKey:@"type"];
}

- (U1FolderNode *)rootFolder;
{
	// Look up a U1FolderNode with us as the volume path and some part of us as its name?
	// If it doesn't exist, create it, and return it.
	U1DataRepository *dataRepository = [U1DataRepository sharedDataRepository];
	NSPredicate *p = [NSPredicate predicateWithFormat:@"resourcePath = %@", self.nodePath];
	NSError *error = nil;
	NSArray *results = [dataRepository resultsForEntityClass:[U1FolderNode class] matchingPredicate:p withSortDescriptors:nil error:&error];
	U1FolderNode *rootFolder = [results lastObject];

	if (rootFolder == nil)
	{
		// Upload
		[dataRepository dispatchBlockWithMainContext:^(NSManagedObjectContext *context) {
			
			U1FolderNode *rootFolder = [U1FolderNode insertInManagedObjectContext:context];
			rootFolder.resourcePath = self.nodePath;
			rootFolder.contentPath = self.contentPath;
			[dataRepository save:NULL];
		}];
	
		results = [dataRepository resultsForEntityClass:[U1FolderNode class] matchingPredicate:p withSortDescriptors:nil error:&error];
		rootFolder = [results lastObject];
	}

	return rootFolder;
}

- (BOOL)isCloudFolder;
{
	return [self.kind isEqualToString:@"udf"];
}

@end
