//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "UIActionSheet+U1Additions.h"

@interface U1ActionSheetBlockDelegate : NSObject <UIActionSheetDelegate>
@property (copy) void (^callbackBlock)(NSInteger clickedButtonIndex);
@property (retain) UIActionSheet *actionSheet;
@end


@implementation UIActionSheet (U1Additions)

- (void)showInView:(UIView *)view withCompletionBlock:(void(^)(NSInteger clickedButtonIndex))completionBlock;
{
	U1ActionSheetBlockDelegate *delegate = [U1ActionSheetBlockDelegate new];
	delegate.actionSheet = self;
	delegate.callbackBlock = completionBlock;
	[self setDelegate:delegate];
	[self showInView:view];
}

@end


@implementation U1ActionSheetBlockDelegate

@synthesize callbackBlock, actionSheet;

- (void)dealloc;
{
	[callbackBlock release];
	[actionSheet release];
	[super dealloc];
}

- (void)actionSheet:(UIActionSheet *)actionSheet didDismissWithButtonIndex:(NSInteger)buttonIndex;
{
	dispatch_async(dispatch_get_main_queue(), ^(void) {
		if (self.callbackBlock)
		{
			self.callbackBlock(buttonIndex);
		}
		[self release];
	});
}

@end