//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "UOHTTPFetchOperation.h"


@interface UOHTTPFetchOperation ()
@property (copy) void (^action)(NSString *, NSError *);
@property (retain) NSMutableData *httpData;
@end


@implementation UOHTTPFetchOperation

@synthesize action, httpData;

+ (UOHTTPFetchOperation*)httpOperationWithRequest:(NSURLRequest*)request action:(void(^)(NSString *responseBody, NSError *error))action;
{
	NSParameterAssert(action != nil);
	
	UOHTTPFetchOperation *operation = [[[self alloc] initWithRequest:request] autorelease];
	operation.action = action;
	
	return operation;
}

- (void)dealloc;
{
	[action release];
	[httpData release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response;
{
	[super connection:connection didReceiveResponse:response];
	self.httpData = [NSMutableData data];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data;
{
	[super connection:connection didReceiveData:data];
	[self.httpData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error;
{
	[super connection:connection didFailWithError:error];
	self.httpData = nil;
	self.action(nil, error);
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection;
{
	[super connectionDidFinishLoading:connection];
	NSError *error = nil;
	NSString *responseBody = nil;
	// These status codes suitably cover 'success' for now, will make
	// this more generic when HTTP fetcher is pulled into shared lib.
	if (self.statusCode == 200 ||
		self.statusCode == 204 ||
		self.statusCode == 302)
	{
		responseBody = [[[NSString alloc] initWithData:self.httpData encoding:NSUTF8StringEncoding] autorelease];
	}
	else
	{
		error = [NSError errorWithDomain:@"UOHTTPFetchOperationErrorDomain" code:self.statusCode userInfo:nil];
	}
	self.action(responseBody, error);
}

@end
