//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "UOJSONFetchOperation.h"


@interface UOJSONFetchOperation ()
@property (copy) void (^action)(NSObject *, NSError *);
@property (retain) NSMutableData *jsonData;
@end


@implementation UOJSONFetchOperation

@synthesize action, jsonData;

+ (UOJSONFetchOperation*)jsonOperationWithRequest:(NSURLRequest*)request action:(void(^)(NSObject *jsonResult, NSError *error))action;
{
	NSParameterAssert(action != nil);
	
	UOJSONFetchOperation *operation = [[[UOJSONFetchOperation alloc] initWithRequest:request] autorelease];
	operation.action = action;
	
	return operation;
}

- (void)dealloc;
{
	[action release];
	[jsonData release];
	[super dealloc];
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response;
{
	[super connection:connection didReceiveResponse:response];
	self.jsonData = [NSMutableData data];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data;
{
	[super connection:connection didReceiveData:data];
	[self.jsonData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error;
{
	[super connection:connection didFailWithError:error];
	self.jsonData = nil;
	self.action(nil, error);
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection;
{
	[super connectionDidFinishLoading:connection];
	NSError *error = nil;
	NSObject *jsonObject = nil;
	if (self.statusCode == 200)
	{
        jsonObject = [NSJSONSerialization JSONObjectWithData:self.jsonData options:0 error:&error];
	}
	else
	{
		error = [NSError errorWithDomain:@"UOJSONFetchOperationErrorDomain" code:self.statusCode userInfo:nil];
	}
	self.action(jsonObject, error);
}

@end
