//
//  Copyright 2013 Canonical Ltd.
//
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.


    // Class under test
#import "U1Hasher.h"

    // Collaborators

    // Test support
#import <SenTestingKit/SenTestingKit.h>

#define HC_SHORTHAND
#import <OCHamcrestIOS/OCHamcrestIOS.h>

#define MOCKITO_SHORTHAND
#import <OCMockitoIOS/OCMockitoIOS.h>

@interface FakeAssetRepresentation : NSObject

- (NSUInteger)getBytes:(uint8_t*)buffer fromOffset:(long long)offset length:(NSUInteger)length error:(NSError **)error;
@end

@implementation FakeAssetRepresentation

- (NSUInteger)getBytes:(uint8_t*)buffer fromOffset:(long long)offset length:(NSUInteger)length error:(NSError **)error{
	memcpy(buffer, "test", 4);
	return 4;
}

- (NSUInteger)size{
	return 4;
}

@end

@interface OneAssetFakeAssetsLibrary : NSObject

- (void)assetForURL:(NSURL*)assetURL resultBlock:(ALAssetsLibraryAssetForURLResultBlock)resultBlock failureBlock:(ALAssetsLibraryAccessFailureBlock)failureBlock;

@end

@implementation OneAssetFakeAssetsLibrary
- (void)assetForURL:(NSURL*)assetURL resultBlock:(ALAssetsLibraryAssetForURLResultBlock)resultBlock failureBlock:(ALAssetsLibraryAccessFailureBlock)failureBlock{
	
	FakeAssetRepresentation *fakeRepresentation = [[[FakeAssetRepresentation alloc] init] autorelease];
	ALAsset *mockAsset = mock([ALAsset class]);
	[given([mockAsset defaultRepresentation]) willReturn:fakeRepresentation];
	
	resultBlock(mockAsset);
	
}

- (id)initWithMockAssetArray:(NSArray *)newMockAssets{
	self = [super init];
	if(!self) return nil;

	return self;
}

@end

@interface U1HasherTest : SenTestCase
@end

@implementation U1HasherTest
{
	NSData *exampleData;
	NSData *expectedContentHash;
	NSData *expectedMagicHash;
}

- (void)setUp{
	exampleData = [[NSData alloc] initWithBytes:"test" length:4];
	expectedContentHash = [[NSData alloc] initWithBytes:"\xa9J\x8f\xe5\xcc\xb1\x9b\xa6\x1cL\x08s\xd3\x91\xe9\x87\x98/\xbb\xd3" length:20];
	expectedMagicHash = [[NSData alloc] initWithBytes:"}J%\xf0!\xe8\xd6\x98kQu\x07\x92H\x06\x84\xfe{z=" length:20];
	[super setUp];
}

- (void)tearDown{
	[exampleData release];
	[expectedContentHash release];
	[expectedMagicHash release];
}

- (void)testContentHash{
	NSData *contentHashData = (NSData *)[U1Hasher contentHashForAssetData:exampleData length:[exampleData length]];
	
	STAssertEqualObjects(contentHashData, expectedContentHash, @"does not match");
}


- (void)testMagicHash{
	NSData *magicHashData = (NSData *)[U1Hasher magicHashForAssetData:exampleData length:[exampleData length]];
	
	STAssertEqualObjects(magicHashData, expectedMagicHash, @"does not match");
}

- (void)testSynchronouslyGetHashes{
	U1Hasher *hasher = [[[U1Hasher alloc] init] autorelease];
	hasher.assetsLibrary = (ALAssetsLibrary *)[[[OneAssetFakeAssetsLibrary alloc] init] autorelease];
	
	// asset url is ignored, can be anything:
	NSDictionary *hashDict = [hasher synchronouslyGetHashesForAssetURL:[NSURL URLWithString:@"http://example.com"]];

	STAssertEqualObjects(hashDict[@"contentHash"], expectedContentHash, @"does not match");
	STAssertEqualObjects(hashDict[@"magicHash"], expectedMagicHash, @"does not match");

	
}

@end
