//
//  Copyright 2011 Canonical Ltd.
//	
//  This program is free software: you can redistribute it and/or modify it
//  under the terms of the GNU Affero General Public License version 3,
//  as published by the Free Software Foundation.
//	
//  This program is distributed in the hope that it will be useful, but
//  WITHOUT ANY WARRANTY; without even the implied warranties of
//  MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
//  PURPOSE.  See the GNU Affero General Public License for more details.
//
//  You should have received a copy of the GNU Affero General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

#import "U1FuzzyDateTransformer.h"


static const NSUInteger kSECONDS_PER_MINUTE = 60;
static const NSUInteger kSECONDS_PER_HOUR = 60 * kSECONDS_PER_MINUTE;
static const NSUInteger kSECONDS_PER_DAY = 24 * kSECONDS_PER_HOUR;
static const NSUInteger kSECONDS_PER_WEEK = 7 * kSECONDS_PER_DAY;
static const NSUInteger kABOUT_A_WEEK = kSECONDS_PER_WEEK + kSECONDS_PER_DAY;


@implementation U1FuzzyDateTransformer

+ (Class)transformedValueClass { return [NSString class]; }

+ (BOOL)allowsReverseTransformation { return NO; }

+ (NSDateFormatter*)dateFormatter;
{
	static NSDateFormatter *dateFormatter = nil;
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^{
		dateFormatter = [[NSDateFormatter alloc] init];
		[dateFormatter setDateStyle:NSDateFormatterShortStyle];
	});
	return dateFormatter;
}

- (id)transformedValue:(id)value;
{
	NSDate *date = value;
	NSTimeInterval interval = -[date timeIntervalSinceNow];
	
	
	NSString *fuzzyDate = nil;
	if (interval < kSECONDS_PER_MINUTE)
	{
		return NSLocalizedString(@"just now", nil);
	}
	else if (interval < kSECONDS_PER_HOUR)
	{
		NSUInteger minutes = interval / kSECONDS_PER_MINUTE;
		NSString *format = (minutes > 1) ? NSLocalizedString(@"%u minutes ago", nil) : NSLocalizedString(@"%u minute ago", nil);
		fuzzyDate = [NSString stringWithFormat:format, minutes];
	}
	else if (interval < kSECONDS_PER_DAY)
	{
		NSUInteger hours = interval / kSECONDS_PER_HOUR;
		NSString *format = (hours > 1) ? NSLocalizedString(@"%u hours ago", nil) : NSLocalizedString(@"%u hour ago", nil);
		fuzzyDate = [NSString stringWithFormat:format, hours];
	}
	else if (interval < kSECONDS_PER_WEEK)
	{
		NSUInteger days = interval / kSECONDS_PER_DAY;
		NSString *format = (days > 1) ? NSLocalizedString(@"%u days ago", nil) : NSLocalizedString(@"%u day ago", nil);
		fuzzyDate = [NSString stringWithFormat:format, days];
	}
	else if (interval < kABOUT_A_WEEK)
	{
		return NSLocalizedString(@"1 week ago", nil);
	}
	else
	{
		fuzzyDate = [[U1FuzzyDateTransformer dateFormatter] stringFromDate:date];
	}
	
    return fuzzyDate;
}

@end
